@@ -23,7 +23,7 @@ use File::Find qw(finddepth);
 use File::Basename qw(dirname);
 use File::Path ();
 use File::Spec::Functions qw(catfile abs2rel splitdir canonpath
-                             catdir file_name_is_absolute);
+                             catdir file_name_is_absolute devnull);
 use Cwd qw(fastcwd);
 
 use Apache::TestConfigPerl ();
@@ -35,30 +35,31 @@ use Socket ();
 use vars qw(%Usage);
 
 %Usage = (
-   top_dir       => 'top-level directory (default is $PWD)',
-   t_dir         => 'the t/ test directory (default is $top_dir/t)',
-   t_conf        => 'the conf/ test directory (default is $t_dir/conf)',
-   t_logs        => 'the logs/ test directory (default is $t_dir/logs)',
-   t_conf_file   => 'test httpd.conf file (default is $t_conf/httpd.conf)',
-   src_dir       => 'source directory to look for mod_foos.so',
-   serverroot    => 'ServerRoot (default is $t_dir)',
-   documentroot  => 'DocumentRoot (default is $ServerRoot/htdocs',
-   port          => 'Port [port_number|select] (default ' . DEFAULT_PORT . ')',
-   servername    => 'ServerName (default is localhost)',
-   user          => 'User to run test server as (default is $USER)',
-   group         => 'Group to run test server as (default is $GROUP)',
-   bindir        => 'Apache bin/ dir (default is apxs -q BINDIR)',
-   sbindir       => 'Apache sbin/ dir (default is apxs -q SBINDIR)',
-   httpd         => 'server to use for testing (default is $bindir/httpd)',
-   target        => 'name of server binary (default is apxs -q TARGET)',
-   apxs          => 'location of apxs (default is from Apache::BuildConfig)',
-   httpd_conf    => 'inherit config from this file (default is apxs derived)',
-   maxclients    => 'maximum number of concurrent clients (default is 1)',
-   perlpod       => 'location of perl pod documents (for testing downloads)',
-   proxyssl_url  => 'url for testing ProxyPass / https (default is localhost)',
-   sslca         => 'location of SSL CA (default is $t_conf/ssl/ca)',
-   sslcaorg      => 'SSL CA organization to use for tests (default is asf)',
-   libmodperl    => 'path to mod_perl\'s .so (full or relative to LIBEXECDIR)',
+   top_dir         => 'top-level directory (default is $PWD)',
+   t_dir           => 'the t/ test directory (default is $top_dir/t)',
+   t_conf          => 'the conf/ test directory (default is $t_dir/conf)',
+   t_logs          => 'the logs/ test directory (default is $t_dir/logs)',
+   t_conf_file     => 'test httpd.conf file (default is $t_conf/httpd.conf)',
+   src_dir         => 'source directory to look for mod_foos.so',
+   serverroot      => 'ServerRoot (default is $t_dir)',
+   documentroot    => 'DocumentRoot (default is $ServerRoot/htdocs',
+   port            => 'Port [port_number|select] (default ' . DEFAULT_PORT . ')',
+   servername      => 'ServerName (default is localhost)',
+   user            => 'User to run test server as (default is $USER)',
+   group           => 'Group to run test server as (default is $GROUP)',
+   bindir          => 'Apache bin/ dir (default is apxs -q BINDIR)',
+   sbindir         => 'Apache sbin/ dir (default is apxs -q SBINDIR)',
+   httpd           => 'server to use for testing (default is $bindir/httpd)',
+   target          => 'name of server binary (default is apxs -q TARGET)',
+   apxs            => 'location of apxs (default is from Apache::BuildConfig)',
+   startup_timeout => 'seconds to wait for the server to start (default is 60)',
+   httpd_conf      => 'inherit config from this file (default is apxs derived)',
+   maxclients      => 'maximum number of concurrent clients (default is 1)',
+   perlpod         => 'location of perl pod documents (for testing downloads)',
+   proxyssl_url    => 'url for testing ProxyPass / https (default is localhost)',
+   sslca           => 'location of SSL CA (default is $t_conf/ssl/ca)',
+   sslcaorg        => 'SSL CA organization to use for tests (default is asf)',
+   libmodperl      => 'path to mod_perl\'s .so (full or relative to LIBEXECDIR)',
    (map { $_ . '_module_name', "$_ module name"} qw(cgi ssl thread access auth)),
 );
 
@@ -232,7 +233,7 @@ sub new {
 
     $vars->{scheme}       ||= 'http';
     $vars->{servername}   ||= $self->default_servername;
-    $vars->{port}           = $self->select_port;
+    $vars->{port}           = $self->select_first_port;
     $vars->{remote_addr}  ||= $self->our_remote_addr;
 
     $vars->{user}         ||= $self->default_user;
@@ -398,7 +399,7 @@ sub add_config {
         $args = join "\n", @$directive;
     }
     else {
-        $args = "$directive " .
+        $args = join " ", grep length($_), $directive,
           (ref($arg) && (ref($arg) eq 'ARRAY') ? "@$arg" : $arg || "");
     }
 
@@ -538,7 +539,7 @@ sub default_servername {
 }
 
 # memoize the selected value (so we make sure that the same port is used
-# via select). The problem is that select_port() is called 3 times after
+# via select). The problem is that select_first_port() is called 3 times after
 # -clean, and it's possible that a lower port will get released
 # between calls, leading to various places in the test suite getting a
 # different base port selection.
@@ -548,7 +549,7 @@ sub default_servername {
 # bind() will actually get the port. So there is a need in another
 # check and reconfiguration just before the server starts.
 #
-sub select_port {
+sub select_first_port {
     my $self = shift;
 
     my $port ||= $ENV{APACHE_PORT} || $self->{vars}{port} || DEFAULT_PORT;
@@ -588,7 +589,12 @@ my $remote_addr;
 sub our_remote_addr {
     my $self = shift;
     my $name = $self->default_servername;
-    $remote_addr ||= Socket::inet_ntoa((gethostbyname($name))[-1]);
+    my $iaddr = (gethostbyname($name))[-1];
+    unless (defined $iaddr) {
+        error "Can't resolve host: '$name' (check /etc/hosts)";
+        exit 1;
+    }
+    $remote_addr ||= Socket::inet_ntoa($iaddr);
 }
 
 sub default_loopback {
@@ -600,7 +606,7 @@ sub port {
 
     unless ($module) {
         my $vars = $self->{vars};
-        return $self->select_port() unless $vars->{scheme} eq 'https';
+        return $self->select_first_port() unless $vars->{scheme} eq 'https';
         $module = $vars->{ssl_module_name};
     }
     return $self->{vhosts}->{$module}->{port};
@@ -1342,10 +1348,14 @@ EOF
 }
 
 sub need_reconfiguration {
-    my $self = shift;
+    my($self, $conf_opts) = @_;
     my @reasons = ();
     my $vars = $self->{vars};
 
+    if (my $port = $conf_opts->{port} || $Apache::TestConfig::Argv{port}) {
+        push @reasons, "'-port $port' requires reconfiguration";
+    }
+
     my $exe = $vars->{apxs} || $vars->{httpd};
     # if httpd.conf is older than executable
     push @reasons, 
@@ -1420,7 +1430,8 @@ sub which {
 sub apxs {
     my($self, $q, $ok_fail) = @_;
     return unless $self->{APXS};
-    my $val = qx($self->{APXS} -q $q 2>/dev/null);
+    my $devnull = devnull();
+    my $val = qx($self->{APXS} -q $q 2>$devnull);
     chomp $val if defined $val; # apxs post-2.0.40 adds a new line
     unless ($val) {
         if ($ok_fail) {
